/* ============================================================
 * File  : channel.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-18
 * Description :
 *
 * A generic Channel
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef CHANNEL_CPP
#define CHANNEL_CPP

// MythTV headers
#include <mythtv/libmythui/mythmainwindow.h>
#include <mythtv/util.h>
#include <mythtv/httpcomms.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythdirs.h>

using namespace std;

#include "channel.h"


Channel::Channel(QString name, QString url, Type type) 
    : QObject()
{

    m_name = name;
    m_url = url;
    m_type = type;

    m_destDir = channelDir();

    m_data.resize(0);

}

Channel::~Channel(){

}

void Channel::ReplaceHtmlChar(QString &s)
{
    s.replace("&amp;", "&");
    s.replace("&lt;", "<");
    s.replace("&gt;", ">");
    s.replace("&quot;", "\"");
    s.replace("&apos;", "\'");
    s.replace("&#8230;",QChar(8230));
    s.replace("&#233;",QChar(233));
    s.replace("&mdash;", QChar(8212));
    s.replace("&nbsp;", " ");
    s.replace("&#160;", QChar(160));
    s.replace("&#225;", QChar(225));
    s.replace("&#8216;", QChar(8216));
    s.replace("&#8217;", QChar(8217));
    s.replace("&#039;", "\'");
    s.replace("&ndash;", QChar(8211));
    // german umlauts
    s.replace("&auml;", QChar(0x00e4));
    s.replace("&ouml;", QChar(0x00f6));
    s.replace("&uuml;", QChar(0x00fc));
    s.replace("&Auml;", QChar(0x00c4));
    s.replace("&Ouml;", QChar(0x00d6));
    s.replace("&Uuml;", QChar(0x00dc));
    s.replace("&szlig;", QChar(0x00df));
}

void Channel::process(){
    QDomDocument domDoc;

    QString sFilename = channelDir()+QString("/")+m_name;

    QFile xmlFile(sFilename);
    if (!xmlFile.exists()) {
        return;
    }

    if (!xmlFile.open(QIODevice::ReadOnly)) {
        VERBOSE(VB_IMPORTANT, "MythTube: TubeEngine: failed to open xmlfile");
        return;
    }

    if (!domDoc.setContent(&xmlFile)) {
        VERBOSE(VB_IMPORTANT, "MythTube: TubeEngine: failed to set content from xmlfile");
        return;
    }

    m_videoList = parseRSS2(domDoc);

    //save image channel

    xmlFile.close();
}

TubeVideo::List Channel::parseRSS2(QDomDocument domDoc)
{
    TubeVideo::List vList;

    QDomNode channelNode = domDoc.documentElement().namedItem(QString::fromLatin1("channel"));

    m_desc = channelNode.namedItem(QString::fromLatin1("description")).toElement().text().simplifyWhiteSpace();

    QDomNode imageNode = channelNode.namedItem(QString::fromLatin1("image"));
    if (!imageNode.isNull())
        m_imageURL = imageNode.namedItem(QString::fromLatin1("url")).toElement().text().simplifyWhiteSpace();

    //*****  http://www.itunes.com/dtds/podcast-1.0.dtd <=> itunes *****//

    if(m_imageURL.isEmpty()){
        QDomNode imageNode = channelNode.namedItem(QString::fromLatin1("itunes:image"));
        if (!imageNode.isNull())
        {
            QDomAttr thumburl = imageNode.toElement().attributeNode("href");
            if (!thumburl.isNull())
                m_imageURL = thumburl.value();
        } else
            m_imageURL = QString::null;
    }

    QDomNodeList items = domDoc.elementsByTagName(QString::fromLatin1("item"));

    QDomNode itemNode;
    QString title, description, url, thumbnail, mediaurl, enclosure, imageURL, enclosure_type;
    for (int i = 0; i < items.count(); i++) {
        itemNode = items.item(i);
        title    = itemNode.namedItem(QString::fromLatin1("title")).toElement().text().simplifyWhiteSpace();
        if (!title.isNull())
            ReplaceHtmlChar(title);

        QDomNode descNode = itemNode.namedItem(QString::fromLatin1("description"));
        if (!descNode.isNull())
        {
            description = descNode.toElement().text().simplifyWhiteSpace();
            ReplaceHtmlChar(description);
            //delete html tags
            QRegExp rx( "<.*>" );
            rx.setMinimal( TRUE );
            description.remove(rx);
        }
        else
            description = QString::null;

        QDomNode linkNode = itemNode.namedItem(QString::fromLatin1("link"));
        if (!linkNode.isNull())
            url = linkNode.toElement().text().simplifyWhiteSpace();
        else
            url = QString::null;

        QDomNode enclosureNode = itemNode.namedItem(QString::fromLatin1("enclosure"));
        if (!enclosureNode.isNull())
        {
            QDomAttr enclosureURL = enclosureNode.toElement().attributeNode("url");
            if (!enclosureURL.isNull())
               enclosure  = enclosureURL.value();

            QDomAttr enclosureType = enclosureNode.toElement().attributeNode("type");
            if (!enclosureType.isNull())
               enclosure_type  = enclosureType.value();

            // VERBOSE(VB_GENERAL, QString("MythNews: Enclosure URL is %1").arg(enclosure));
        } else
            enclosure = QString::null;

        // From this point forward, we process RSS 2.0 media tags.  Please put all
        // other tag processing before this


        //****************************************************//
        //************      Namespace parse     **************//
        //****************************************************//


        //**********           Namespace            **********//
        //*****  http://search.yahoo.com/mrss/ <=> media *****//
        //****************************************************//

        //Note: other site how youtube use media:thumbnail on mediar:group but I don't now what is correct
        QDomNode thumbNode = itemNode.namedItem(QString::fromLatin1("media:thumbnail"));
        if (!thumbNode.isNull())
        {
            QDomAttr thumburl = thumbNode.toElement().attributeNode("url");
            if (!thumburl.isNull())
                thumbnail = thumburl.value();
            // VERBOSE(VB_GENERAL, QString("MythNews: Thumbnail is %1").arg(thumbnail));
        } else
            thumbnail = QString::null;

        QDomNode playerNode = itemNode.namedItem(QString::fromLatin1("media:player"));
        if (!playerNode.isNull())
        {
            QDomAttr mediaURL = playerNode.toElement().attributeNode("url");
            if (!mediaURL.isNull())
               mediaurl  = mediaURL.value();
            // VERBOSE(VB_GENERAL, QString("MythNews: Media URL is %1").arg(mediaurl));
        } else
            mediaurl = QString::null;

        // If present, the media:description superscedes the RSS description
        descNode = itemNode.namedItem(QString::fromLatin1("media:description"));
        if (!descNode.isNull())
            description = descNode.toElement().text().simplifyWhiteSpace();

        // Some media: tags can be enclosed in a media:group item.  If this item is
        // present, use it to find the media tags, otherwise, proceed.
        QDomNode mediaGroup = itemNode.namedItem(QString::fromLatin1("media:group"));
        if (!mediaGroup.isNull())
            itemNode = mediaGroup;

        if (enclosure.isEmpty())
        {
            QDomNode contentNode = itemNode.namedItem(QString::fromLatin1("media:content"));
            if (!contentNode.isNull())
            {
                QDomAttr enclosureURL = contentNode.toElement().attributeNode("url");
                if (!enclosureURL.isNull())
                   enclosure  = enclosureURL.value();

                QDomAttr enclosureType = contentNode.toElement().attributeNode("type");
                if (!enclosureType.isNull())
                   enclosure_type  = enclosureType.value();
                // VERBOSE(VB_GENERAL, QString("MythNews: media:content URL is %1").arg(enclosure));
            }
        }

        //If exist media group this regenerate itemNode
        itemNode = items.item(i);


        //*********                  Namespace                     *********//
        //*****  http://www.itunes.com/dtds/podcast-1.0.dtd <=> itunes *****//
        //******************************************************************//

        if(thumbnail.isEmpty()){
            QDomNode thumbNode = itemNode.namedItem(QString::fromLatin1("itunes:image"));
            if (!thumbNode.isNull())
            {
                QDomAttr thumburl = thumbNode.toElement().attributeNode("href");
                if (!thumburl.isNull())
                    thumbnail = thumburl.value();
                VERBOSE(VB_GENERAL, QString("MythNews: Thumbnail is %1").arg(thumbnail));
            } else
                thumbnail = QString::null;
        }

        if(description.isEmpty()){
            QDomNode descNode = itemNode.namedItem(QString::fromLatin1("itunes:summary"));
            if (!descNode.isNull())
            {
                description = descNode.toElement().text().simplifyWhiteSpace();
                ReplaceHtmlChar(description);
                //delete html tags
                QRegExp rx( "<.*>" );
                rx.setMinimal( TRUE );
                description.remove(rx);
            }
            else
                description = QString::null;
        }




        vList.append(new TubeVideo(title, description, url, thumbnail, mediaurl, enclosure,"","",""));
    }

    return vList;
}
TubeVideo::List Channel::parseRSS(QDomDocument domDoc)
{
    TubeVideo::List vList;

    QDomNode channelNode = domDoc.documentElement().namedItem(QString::fromLatin1("channel"));

    m_desc = channelNode.namedItem(QString::fromLatin1("description")).toElement().text().simplifyWhiteSpace();

    QDomNode imageNode = channelNode.namedItem(QString::fromLatin1("image"));
    if (!imageNode.isNull())
        m_imageURL = imageNode.namedItem(QString::fromLatin1("url")).toElement().text().simplifyWhiteSpace();

    QDomNodeList items = domDoc.elementsByTagName(QString::fromLatin1("item"));

    QDomNode itemNode;
    QString title, description, url, thumbnail, mediaurl, enclosure, imageURL, enclosure_type;
    for (int i = 0; i < items.count(); i++) {
        itemNode = items.item(i);
        title    = itemNode.namedItem(QString::fromLatin1("title")).toElement().text().simplifyWhiteSpace();
        if (!title.isNull())
            ReplaceHtmlChar(title);

        QDomNode descNode = itemNode.namedItem(QString::fromLatin1("description"));
        if (!descNode.isNull())
        {
            description = descNode.toElement().text().simplifyWhiteSpace();
            ReplaceHtmlChar(description);
            //delete html tags
            QRegExp rx( "<.*>" );
            rx.setMinimal( TRUE );
            description.remove(rx);
        }
        else
            description = QString::null;

        QDomNode linkNode = itemNode.namedItem(QString::fromLatin1("link"));
        if (!linkNode.isNull())
            url = linkNode.toElement().text().simplifyWhiteSpace();
        else
            url = QString::null;

        QDomNode enclosureNode = itemNode.namedItem(QString::fromLatin1("enclosure"));
        if (!enclosureNode.isNull())
        {
            QDomAttr enclosureURL = enclosureNode.toElement().attributeNode("url");
            if (!enclosureURL.isNull())
               enclosure  = enclosureURL.value();

            QDomAttr enclosureType = enclosureNode.toElement().attributeNode("type");
            if (!enclosureType.isNull())
               enclosure_type  = enclosureType.value();

            // VERBOSE(VB_GENERAL, QString("MythNews: Enclosure URL is %1").arg(enclosure));
        } else
            enclosure = QString::null;

//this is a patch for la dosis

QDomNode thumbNode = itemNode.namedItem(QString::fromLatin1("media:thumbnail"));
if (!thumbNode.isNull())
{
    QDomAttr thumburl = thumbNode.toElement().attributeNode("url");
    if (!thumburl.isNull())
        thumbnail = thumburl.value();
    VERBOSE(VB_GENERAL, QString("MythNews: Thumbnail is %1").arg(thumbnail));
} else
    thumbnail = QString::null;


        // From this point forward, we process RSS 2.0 media tags.  Please put all
        // other tag processing before this
        // Some media: tags can be enclosed in a media:group item.  If this item is
        // present, use it to find the media tags, otherwise, proceed.
        QDomNode mediaGroup = itemNode.namedItem(QString::fromLatin1("media:group"));
        if (!mediaGroup.isNull())
            itemNode = mediaGroup;



/*
        QDomNode thumbNode = itemNode.namedItem(QString::fromLatin1("media:thumbnail"));
        if (!thumbNode.isNull())
        {
            QDomAttr thumburl = thumbNode.toElement().attributeNode("url");
            if (!thumburl.isNull())
                thumbnail = thumburl.value();
            VERBOSE(VB_GENERAL, QString("MythNews: Thumbnail is %1").arg(thumbnail));
        } else
            thumbnail = QString::null;*/







        QDomNode playerNode = itemNode.namedItem(QString::fromLatin1("media:player"));
        if (!playerNode.isNull())
        {
            QDomAttr mediaURL = playerNode.toElement().attributeNode("url");
            if (!mediaURL.isNull())
               mediaurl  = mediaURL.value();
            // VERBOSE(VB_GENERAL, QString("MythNews: Media URL is %1").arg(mediaurl));
        } else
            mediaurl = QString::null;

        // If present, the media:description superscedes the RSS description
        descNode = itemNode.namedItem(QString::fromLatin1("media:description"));
        if (!descNode.isNull())
            description = descNode.toElement().text().simplifyWhiteSpace();

        if (enclosure.isEmpty())
        {
            QDomNode contentNode = itemNode.namedItem(QString::fromLatin1("media:content"));
            if (!contentNode.isNull())
            {
                QDomAttr enclosureURL = contentNode.toElement().attributeNode("url");
                if (!enclosureURL.isNull())
                   enclosure  = enclosureURL.value();

                QDomAttr enclosureType = contentNode.toElement().attributeNode("type");
                if (!enclosureType.isNull())
                   enclosure_type  = enclosureType.value();
                // VERBOSE(VB_GENERAL, QString("MythNews: media:content URL is %1").arg(enclosure));

                // This hack causes the engine to prefer other content over MP4 encoded content
                // because this can often include AAC audio which requires additional compile options
                // to enable in MythTV Internal player
                /*
                while (enclosure_type == "video/mp4")
                {
                    QDomNode altcontentNode = contentNode.nextSibling();
                    if (altcontentNode.nodeName() != "media:content")
                        break;

                    QDomAttr enclosureType = altcontentNode.toElement().attributeNode("type");
                    if (!enclosureType.isNull())
                    {
                        enclosure_type  = enclosureType.value();

                        QDomAttr altenclosureURL = altcontentNode.toElement().attributeNode("url");
                        if (!altenclosureURL.isNull())
                            enclosure  = altenclosureURL.value();
                    }
                }
                */
            }
        }

        vList.append(new TubeVideo(title, description, url, thumbnail, mediaurl, enclosure,"","",""));
    }

    return vList;
}

QString Channel::channelDir (QString path) {

    QString fileprefix = GetConfDir();

    QDir dir(fileprefix);
    if (!dir.exists())
        dir.mkdir(fileprefix);
    fileprefix += "/MythTube";
    dir = QDir(fileprefix);
    if (!dir.exists())
        dir.mkdir(fileprefix);
    fileprefix += "/" + m_name;
    dir = QDir(fileprefix);
    if (!dir.exists())
        dir.mkdir(fileprefix);

    if(!path.isEmpty()){
        QStringList dir_list = QStringList::split("/" ,path);
        for(int i = 0; i < dir_list.count(); i++){
            fileprefix += "/" + dir_list[i];
            QDir dir_aux(fileprefix);
            if (!dir_aux.exists())
                dir_aux.mkdir(fileprefix);
        }
    }

    return fileprefix;
}

#endif
