/* ============================================================
 * File  : channeleditor.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-18
 * Description :
 *
 * Interface to edit channels
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef CHANNELSEDITOR_CPP
#define CHANNELSEDITOR_CPP

// C Headers
#include <unistd.h>
#include <math.h>

// QT headers
#include <qapplication.h>
#include <q3network.h>
#include <qdir.h>

//Added by qt3to4:
#include <QUrl>

// MythTV headers
#include <mythtv/libmythui/mythmainwindow.h>
#include <mythtv/util.h>
#include <mythtv/httpcomms.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythdirs.h>

using namespace std;

#include "channeleditor.h"
#include "tubedbutil.h"

#include <qfile.h>
#include <qstringlist.h>
#include <qdatastream.h>

// 

//========================================================================================
//              Constructor, Desctructor and setup function
//========================================================================================

ChannelsEditor::ChannelsEditor(Channel *channel, bool edit, MythScreenStack *parent,
                   const char *name)
    : MythScreenType (parent, name)
{


    m_titleText = m_nameLabelText = m_urlLabelText = NULL;
    m_nameEdit = m_urlEdit = NULL;
    m_okButton = m_cancelButton = NULL;

    m_channel = channel;
    m_editing = edit;

    if (m_editing)
        m_channelName = m_channel->name();
    else
        m_channelName = "";

}

ChannelsEditor::~ChannelsEditor() { }

bool ChannelsEditor::Create(){

    bool foundtheme = false;

    // Load the theme for this screen
    foundtheme = LoadWindowFromXML("tube-ui.xml", "editor", this);

    if (!foundtheme)
        return false;

    m_titleText = dynamic_cast<MythUIText *> (GetChild("title"));

    if (m_titleText)
    {
      if (m_editing)
          m_titleText->SetText(tr("Edit Site Details"));
      else
          m_titleText->SetText(tr("Enter Site Details"));
    }

    m_nameLabelText = dynamic_cast<MythUIText *> (GetChild("namelabel"));
    m_urlLabelText = dynamic_cast<MythUIText *> (GetChild("urllabel"));

    m_nameEdit = dynamic_cast<MythUITextEdit *> (GetChild("name"));
    m_urlEdit = dynamic_cast<MythUITextEdit *> (GetChild("url"));

    m_okButton = dynamic_cast<MythUIButton *> (GetChild("ok"));
    m_cancelButton = dynamic_cast<MythUIButton *> (GetChild("cancel"));

    if (!m_nameEdit || !m_urlEdit  || !m_okButton
        || !m_cancelButton)
    {
        VERBOSE(VB_IMPORTANT, "Theme is missing critical theme elements.");
        return false;
    }

    if (m_nameLabelText)
        m_nameLabelText->SetText(tr("Name:"));
    if (m_urlLabelText)
        m_urlLabelText->SetText(tr("URL:"));

    m_okButton->SetText(tr("Ok"));
    m_cancelButton->SetText(tr("Cancel"));

    connect(m_okButton, SIGNAL(Clicked()), this, SLOT(Save()));
    connect(m_cancelButton, SIGNAL(Clicked()), this, SLOT(Exit()));

    if (m_editing)
    {
        m_nameEdit->SetText(m_channel->name());
        m_urlEdit->SetText(m_channel->url());
    }

    if (!BuildFocusList())
        VERBOSE(VB_IMPORTANT, "Failed to build a focuslist. Something is wrong");

    SetFocusWidget(m_nameEdit);

    return true;
}


bool ChannelsEditor::keyPressEvent(QKeyEvent *event)
{
    if (GetFocusWidget()->keyPressEvent(event))
        return true;

    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Tube", event, actions);

    if (!handled && MythScreenType::keyPressEvent(event))
        handled = true;

    return handled;
}


void ChannelsEditor::Exit()
{
    GetScreenStack()->PopScreen();
}

void ChannelsEditor::Save()
{

    if (m_editing && m_channelName != "")
        removeFromDB(m_channelName);

    insertInDB(m_nameEdit->GetText(), m_urlEdit->GetText(), Channel::Feed);

    Exit();
}

#endif
