/* ============================================================
 * File  : feedchannel.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-18
 * Description :
 *
 * A Feed Channel
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef FEEDCHANNEL_CPP
#define FEEDCHANNEL_CPP

// C Headers
#include <unistd.h>
#include <math.h>

// QT headers
#include <qapplication.h>
#include <q3network.h>
#include <qdir.h>

//Added by qt3to4:
#include <QUrl>

// MythTV headers
#include <mythtv/libmythui/mythmainwindow.h>
#include <mythtv/util.h>
#include <mythtv/httpcomms.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>
#include <mythtv/mythdirs.h>

using namespace std;

#include "feedchannel.h"

#include <qfile.h>
#include <qstringlist.h>
#include <qdatastream.h>

//========================================================================================
//              Constructor, Desctructor and setup function
//========================================================================================

FeedChannel::FeedChannel(QString name, QString url) : Channel(name, url, Feed)
{
    m_name = name;
    m_url = url;

    m_destDir = channelDir();

    m_data.resize(0);

    q3InitNetworkProtocols();
    m_urlOp = new Q3UrlOperator(m_url);

    connect(m_urlOp, SIGNAL(data(const QByteArray&, Q3NetworkOperation*)),
            this, SLOT(slotGotData(const QByteArray&, Q3NetworkOperation*)));
    connect(m_urlOp, SIGNAL(finished(Q3NetworkOperation*)),
            this, SLOT(slotFinished(Q3NetworkOperation*)));

}

FeedChannel::~FeedChannel(){}

void FeedChannel::retrieve(){

    m_data.resize(0);
    m_videoList.clear();
    m_urlOp->get(m_url);

}

void FeedChannel::slotFinished(Q3NetworkOperation* op)
{

    if (op->state() == Q3NetworkProtocol::StDone &&
        op->errorCode() == Q3NetworkProtocol::NoError)
    {
        QFile xmlFile(channelDir()+QString("/")+m_name);
        if (xmlFile.open( QIODevice::WriteOnly )) {
            QDataStream stream( &xmlFile );
            stream.writeRawBytes(m_data.data(), m_data.size());
            xmlFile.close();
        }
        else
        {
            VERBOSE(VB_IMPORTANT, "FeedChannel: FeedChannel: Write failed on: ");
            VERBOSE(VB_IMPORTANT, channelDir());
        }
    }
    //get error
    else if(op->errorCode() != Q3NetworkProtocol::NoError){
        //need save error on a code or string and proporcionate a funtion to get it from screen to info the user
        VERBOSE(VB_IMPORTANT, "FeedChannel: FeedChannel: Network ERROR!!!! CODE: ");
        VERBOSE(VB_IMPORTANT, "CODE: " + QString::number( op->errorCode() ) );
    }

    emit finished(this);
}

void FeedChannel::slotGotData(const QByteArray& data,
                           Q3NetworkOperation* op)
{
    if (op)
    {
        const char *charData = data.data();
        int len = data.count();

        int size = m_data.size();
        m_data.resize(size + len);
        memcpy(m_data.data()+size, charData, len);
    }else {
        VERBOSE(VB_IMPORTANT, "FeedChannel: FeedChannel: NO NETWORK OPERATOR");
    }
}

#endif
