/* ============================================================
 * File  : mythtubeguide.cpp
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-26
 * Description :
 *
 * MythTube online channel guide, MiroGuide
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef MYTHTUBEGUIDE_CPP
#define MYTHTUBEGUIDE_CPP

#include <mythtv/mythcontext.h>
#include <mythtv/mythdirs.h>

#include "mythtubeguide.h"
#include "tubedbutil.h"

#include <string>
#include <iostream>

using namespace std;

class Result{
    public:
        QString cad;
        QString des;
};

MythTubeGuide::MythTubeGuide(MythScreenStack *parent, const char *name)
    : MythScreenType(parent, name)
{

}

MythTubeGuide::~MythTubeGuide(){

}

bool MythTubeGuide::Create(void)
{
    bool foundtheme = false;

    foundtheme = LoadWindowFromXML("tube-ui.xml", "guide", this);

    if (!foundtheme)
        return false;

    m_scriptProcess = new Q3Process();

    m_searchEdit = dynamic_cast<MythUITextEdit *> (GetChild("searchedit"));
    m_logoImage = dynamic_cast<MythUIImage *> (GetChild("logoimage"));
    m_searchButton = dynamic_cast<MythUIButton *> (GetChild("search"));
    m_categoriesButton = dynamic_cast<MythUIButton *> (GetChild("categories"));
    m_languagesButton = dynamic_cast<MythUIButton *> (GetChild("languages"));
    m_popularButton = dynamic_cast<MythUIButton *> (GetChild("popular"));
    m_topratedButton = dynamic_cast<MythUIButton *> (GetChild("toprated"));
    m_resultList = dynamic_cast<MythUIButtonList *> (GetChild("resultlist"));
    m_description = dynamic_cast<MythUIText *> (GetChild("description"));

    if ( !m_logoImage || !m_searchEdit || !m_searchButton || !m_resultList
         || !m_categoriesButton || !m_languagesButton || !m_popularButton || !m_topratedButton || !m_description)
    {
        VERBOSE(VB_IMPORTANT, "Theme is missing critical theme elements.");
        return false;
    }

    m_searchButton->SetText(tr("Search"));
    m_categoriesButton->SetText(tr("Categories"));
    m_languagesButton->SetText(tr("Language"));
    m_popularButton->SetText(tr("Popular"));
    m_topratedButton->SetText(tr("Top Rated"));

    if (!BuildFocusList())
        VERBOSE(VB_IMPORTANT, "Failed to build a focuslist. Something is wrong");

    processReturnType = NONE;
    m_resultList->SetActive(false);

    connect(m_scriptProcess, SIGNAL(processExited()), SLOT(slotProcessScript()));
    connect(m_searchButton, SIGNAL(Clicked()), this, SLOT(search()));
    connect(m_languagesButton, SIGNAL(Clicked()), this, SLOT(showLanguages()));
    connect(m_categoriesButton, SIGNAL(Clicked()), this, SLOT(showCategorys()));
    connect(m_popularButton, SIGNAL(Clicked()), this, SLOT(showPopular()));
    connect(m_topratedButton, SIGNAL(Clicked()), this, SLOT(showTopRated()));
    connect(m_resultList, SIGNAL(itemClicked( MythUIButtonListItem*)), this, SLOT(itemClicked(MythUIButtonListItem*)));
    connect(m_resultList, SIGNAL(itemSelected( MythUIButtonListItem*)), this, SLOT(selectedItem(MythUIButtonListItem*)));

    SetFocusWidget(m_searchEdit);

    return true;

}

bool MythTubeGuide::loadData(){

    return true;
}

void MythTubeGuide::launchMiroScript(QString parameter){

    QString script = "miroguide.py " + parameter;

    QString cmd = QDir::cleanDirPath(QString("%1/%2")
                    .arg(GetShareDir())
                    .arg("mythtube/scripts/"+script));

    QStringList args;
    args += script;

    QStringList split_args = QStringList::split(' ', cmd);

    m_scriptProcess->clearArguments();
    m_scriptProcess->setArguments(split_args);

    VERBOSE(VB_IMPORTANT, "Launch: "+cmd);

    if(!m_scriptProcess->start()){
        VERBOSE(VB_IMPORTANT, "ERROR: Script");
    }
}

void MythTubeGuide::toggleItem(MythUIButtonListItem *item)
{
    if (!item || !item->getData())
        return;

    ChannelItem* site = (ChannelItem*) item->getData();

    bool checked = (item->state() == MythUIButtonListItem::FullChecked);

    if (!checked) {
        if (insertInDB(site->name, site->url, Channel::Feed))
        {
            item->setChecked(MythUIButtonListItem::FullChecked);
        }
    }
    else {
        if (removeFromDB(site->name))
        {
            item->setChecked(MythUIButtonListItem::NotChecked);
        }
    }
}

bool MythTubeGuide::keyPressEvent(QKeyEvent *event)
{
    if (GetFocusWidget()->keyPressEvent(event))
        return true;

    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Tube", event, actions);

    if (!handled && MythScreenType::keyPressEvent(event))
        handled = true;

    return handled;
}

void MythTubeGuide::search(){
    VERBOSE(VB_IMPORTANT, "SEARCH");
    processReturnType = CHANNELS;
    getChannelListBySearch(m_searchEdit->GetText());
}

void MythTubeGuide::getChannelListBySearch(QString query){
    QString qu = query.replace(" ", "+");
    launchMiroScript("-s " + qu);
}

void MythTubeGuide::showCategorys(){
    VERBOSE(VB_IMPORTANT, "CATEGORYS");
    processReturnType = CATEGORIES;
    launchMiroScript("-c");
}

void MythTubeGuide::showLanguages(){
    VERBOSE(VB_IMPORTANT, "LANGUAGES");
    processReturnType = LANGUAGES;
    launchMiroScript("-l");
}

void MythTubeGuide::showPopular(){
    VERBOSE(VB_IMPORTANT, "Populated");
    processReturnType = CHANNELS;
    launchMiroScript("-p");
}

void MythTubeGuide::showTopRated(){
    VERBOSE(VB_IMPORTANT, "TOPRATED");
    processReturnType = CHANNELS;
    launchMiroScript("-t");
}

void MythTubeGuide::slotProcessScript(){
    VERBOSE(VB_IMPORTANT, "Script Finished");

    QString data = m_scriptProcess->readStdout();
    QStringList split = data.split('\n');


    m_resultList->Reset();
    if(processReturnType != CHANNELS){
        for(int i = 0; i < split.size() - 1; i+=2){
            Result *rs = new Result();
            rs->cad = split[i+1];
            rs->des = split[1];
            MythUIButtonListItem* item = new MythUIButtonListItem(m_resultList, split[i]);
            item->setData(rs);
        }
    }else{
        for(int i = 0; i < split.size() - 1; i+=3){
            Result *rs = new Result();
            rs->cad = split[i+1];
            rs->des = split[i+2];
            MythUIButtonListItem* item = new MythUIButtonListItem(m_resultList, split[i], 0, true,
                                    findInDB(split[i]) ?
                                    MythUIButtonListItem::FullChecked :
                                    MythUIButtonListItem::NotChecked);
            item->setData(rs);
        }
    }
}

void MythTubeGuide::getChannelListByCategory(QString category){
    QString ct = category.replace(" ", "+");
    VERBOSE(VB_IMPORTANT, "GETCHANNELLISTBY_CATEGORY " + ct);
    processReturnType = CHANNELS;
    launchMiroScript("-LC " + ct );
}

void MythTubeGuide::getChannelListByLanguage(QString language){
    QString ln = language.replace(" ", "+");
    VERBOSE(VB_IMPORTANT, "GETCHANNELLISTBY_LANGUAGE " + ln);
    processReturnType = CHANNELS;
    launchMiroScript("-LL " + ln );
}

void MythTubeGuide::getChannel(QString id){
    VERBOSE(VB_IMPORTANT, "GETCHANNEL " + id);
    launchMiroScript("-C " + id);
}

void MythTubeGuide::selectChannel(MythUIButtonListItem* item){
    if (!item || !item->getData())
        return;

    Result* rs = (Result*) item->getData();

    bool checked = (item->state() == MythUIButtonListItem::FullChecked);

    if (!checked) {
        if (insertInDB(item->text(), rs->cad, Channel::Feed))
        {
            item->setChecked(MythUIButtonListItem::FullChecked);
        }
    }
    else {
        if (removeFromDB(item->text()))
        {
            item->setChecked(MythUIButtonListItem::NotChecked);
        }
    }
}

void MythTubeGuide::selectedItem(MythUIButtonListItem* item){
    Result* rs = (Result*) item->getData();

    if(rs){
        if(processReturnType == CHANNELS){
            m_description->SetText(rs->des);
        }else{
            m_description->SetText("");
        }
    }
}

void MythTubeGuide::itemClicked(MythUIButtonListItem* item){
    VERBOSE(VB_IMPORTANT, "SELECTED");

    Result* rs = (Result*) item->getData();

    switch(processReturnType){
        case NONE:
            break;
        case CHANNELS:
            VERBOSE(VB_IMPORTANT, "obtener el canal");
            selectChannel(item);
            break;
        case LANGUAGES:
            VERBOSE(VB_IMPORTANT, "obtener lista canales por lenguaje");
            getChannelListByLanguage(rs->cad);
            break;
        case CATEGORIES:
            VERBOSE(VB_IMPORTANT, "obtener lista canales por categoria");
            getChannelListByCategory(rs->cad);
            break;
    }
}


#endif