/* ============================================================
 * File  : mythtubeguide.h
 * Author: Alvaro Gonzalez Sola <cojonuo@gmail.com>
 * Date  : 2008-08-26
 * Description :
 *
 * MythTube online channel guide, MiroGuide
 *
 * Copyright 2008 by Alvaro Gonzalez Sola
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published bythe Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef MYTHTUBEGUIDE_H
#define MYTHTUBEGUIDE_H

#include <mythtv/libmythui/mythscreentype.h>
#include <mythtv/libmythui/mythuibuttonlist.h>

#include <Q3PtrList>
#include <Q3Process>

#include "feedchannel.h"

class MythTubeGuide : public MythScreenType{

Q_OBJECT

    public:

        struct ChannelItem{
            QString name;
            QString url;
            QString lang;
        };

        typedef Q3PtrList<ChannelItem> ChannelList;

        struct ChannelsCategory{
            QString name;
            ChannelList channels;
        };

        typedef Q3PtrList<ChannelsCategory> CategoryList;

        MythTubeGuide(MythScreenStack *parent, const char *name = 0);
        ~MythTubeGuide();

        bool Create(void);
        bool keyPressEvent(QKeyEvent *event);

        bool loadData();

    private:

        enum ScriptType {NONE, LANGUAGES, CATEGORIES, CHANNELS} processReturnType;

        void launchMiroScript(QString parameter);

        CategoryList *categoryList;

        Q3Process* m_scriptProcess;

        MythUIImage *m_logoImage;
        MythUITextEdit *m_searchEdit;
        MythUIButton * m_searchButton;
        MythUIButton * m_categoriesButton;
        MythUIButton * m_languagesButton;
        MythUIButton * m_popularButton;
        MythUIButton * m_topratedButton;
        MythUIButtonList *m_resultList;
        MythUIButtonList *m_categoryList;
        MythUIButtonList *m_channelList;
        MythUIText *m_description;

        void selectChannel(MythUIButtonListItem* item);

        void getChannelListBySearch(QString query);
        void getChannelListByCategory(QString category);
        void getChannelListByLanguage(QString language);

    private slots:

        //repasar estas funciones y ponerles nombres mas apropiados
        void showLanguages();
        void showCategorys();
        void slotProcessScript();
        void search();
        void showPopular();
        void showTopRated();
        void getChannel(QString id);

        void itemClicked(MythUIButtonListItem* item);
        void selectedItem(MythUIButtonListItem* item);

        void toggleItem(MythUIButtonListItem *item);

};

#endif